{***************************************************************************}
{ TUpdateProvider Component                                                 }
{ for Delphi
{ version 1.1
{                                                                           }
{ written by Jeong Byoung Sun                                               }
{            copyright  2004 - 2011                                        }
{            Email : hanuleyewa@paran.com                                   }
{                                                                           }
{ The source code is given as is. The author is not responsible             }
{ for any possible damage done due to the use of this code.                 }
{ The component can be freely used in any application. The complete         }
{ source code remains property of the author and may not be distributed,    }
{ published, given or sold in any form as such. No parts of the source      }
{ code can be included in any other component or application without        }
{ written authorization of the author.                                      }
{***************************************************************************}

unit UpdateProvider;

interface

uses
  Classes, SysUtils, Provider, DB, DBClient, SqlExpr, UpdateSQLS;

type
  TCustomUpdateProvider = class;

  TCustomUpdateProviderSQL = class(TCustomdbxUpdateSQL)
  private
    FUpdateProvider: TCustomUpdateProvider;
  protected
    function GetConnection: TSQLConnection; override;
  published
    property ModifySQL;
    property InsertSQL;
    property DeleteSQL;
    property BeforeGetParam;
    property BeforeUpdate;
    property TransDesc;
  end;

  {TCustomUpdateProvider}
  TCustomUpdateProvider = class(TDataSetProvider)
  private
    FCommitMode: TCommitMode;
    FUpdateSQL: TCustomUpdateProviderSQL;
    FBeforeUpdateRecord: TBeforeUpdateRecordEvent;
    procedure InternalBeforeUpdateRecord(Sender: TObject; SourceDS: TDataSet;
      DeltaDS: TCustomClientDataSet; UpdateKind: TUpdateKind;
      var Applied: Boolean);
  protected
    function InternalApplyUpdates(const Delta: OleVariant; MaxErrors: Integer;
      out ErrorCount: Integer): OleVariant; override;
    property CommitMode: TCommitMode read FCommitMode write FCommitMode default cmNone;
    property UpdateSQL: TCustomUpdateProviderSQL read FUpdateSQL;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published

  end;

  TUpdateProvider = class(TCustomUpdateProvider)
  published
    property CommitMode;
    property UpdateSQL;
  end;

  TRemoteProvider = class(TCustomUpdateProvider)
  private
    FUserParams: TParams;
    procedure BeforeGetUserParam(const AParamName: String; AField: TField;
        const Old: Boolean; var AValue: Variant);
    procedure InternalBeforeApplyUpdates(Sender: TObject; var OwnerData: OleVariant);
  protected

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property UserParams: TParams read FUserParams write FUserParams;
  end;

implementation

uses
  Variants;

{ TCustomUpdateProvider }

procedure TCustomUpdateProvider.InternalBeforeUpdateRecord(Sender: TObject;
  SourceDS: TDataSet; DeltaDS: TCustomClientDataSet;
  UpdateKind: TUpdateKind; var Applied: Boolean);
begin
  if UpdateSQL.SQL[UpdateKind].Text = '' then
    Exit;
  Applied:= False;
  FUpdateSQL.DataSet:= DeltaDS;

  if FCommitMode = cmRecord then
    FUpdateSQL.StartTransaction;
  try
    FUpdateSQL.Apply(UpdateKind);
    if FCommitMode = cmRecord then
      FUpdateSQL.Commit;
    Applied:= True;
  except
    on E: Exception do
    begin
      if FCommitMode = cmRecord then
        FUpdateSQL.Rollback;
      Raise ExceptionClass(E.ClassType).Create(E.Message);
    end;
  end;
end;

constructor TCustomUpdateProvider.Create(AOwner: TComponent);
begin
  inherited;
  FCommitMode:= cmNone;

  FUpdateSQL:= TCustomUpdateProviderSQL.Create(Self);
  FUpdateSQL.Name := Self.Name + 'InternalUpdateSQL1';			{ Do not localize }
  FUpdateSQL.FUpdateProvider:= Self;
  FUpdateSQL.SetSubComponent(True);
end;

destructor TCustomUpdateProvider.Destroy;
begin
  inherited;
end;

function TCustomUpdateProvider.InternalApplyUpdates(const Delta: OleVariant;
  MaxErrors: Integer; out ErrorCount: Integer): OleVariant;
begin
  if FCommitMode = cmPage then
    FUpdateSQL.StartTransaction;

  FBeforeUpdateRecord:= BeforeUpdateRecord;
  BeforeUpdateRecord:= InternalBeforeUpdateRecord;
  try
    try
      Result:= inherited InternalApplyUpdates(Delta, MaxErrors, ErrorCount);
      if FCommitMode = cmPage then
        FUpdateSQL.Commit;
    except
      on E: Exception do
      begin
        if FCommitMode = cmPage then
          FUpdateSQL.Rollback;
        Raise ExceptionClass(E.ClassType).Create(E.Message);
      end;
    end;
  finally
    BeforeUpdateRecord:= FBeforeUpdateRecord;
  end;
end;

{ TProviderUpdateSQL }

function TCustomUpdateProviderSQL.GetConnection: TSQLConnection;
begin
  if Assigned(FUpdateProvider.DataSet) and (FUpdateProvider.DataSet is TCustomSQLDataSet) then
    Result:= TCustomSQLDataSet(FUpdateProvider.DataSet).SQLConnection
  else
    Result:= nil;
end;

{ TRemoteProvider }

procedure TRemoteProvider.BeforeGetUserParam(const AParamName: String;
  AField: TField; const Old: Boolean; var AValue: Variant);
var
  pr: TParam;
begin
  if Old then
    Exit;
  pr:= UserParams.FindParam(AParamName);
  if pr = nil then
    Exit;
  AValue:= pr.Value;
end;

constructor TRemoteProvider.Create(AOwner: TComponent);
begin
  inherited;
  BeforeApplyUpdates:= InternalBeforeApplyUpdates;
  FUserParams:= TParams.Create;
end;

destructor TRemoteProvider.Destroy;
begin
  FUserParams.Free;
  inherited;
end;

procedure TRemoteProvider.InternalBeforeApplyUpdates(Sender: TObject;
  var OwnerData: OleVariant);
var
  uk: TUpdateKind;
  i: Integer;
begin
  FCommitMode:= cmNone;
  if Not VarIsArray(OwnerData) then
  begin
    for uk:= Low(TUpdateKind) to High(TUpdateKind) do
      FUpdateSQL.SQL[uk].Text:= '';
    Exit;
  end;
  FUserParams.Clear;
  UpdateSQL.BeforeGetParam:= nil;
  FCommitMode:= OwnerData[3];
  for uk:= Low(TUpdateKind) to High(TUpdateKind) do
    FUpdateSQL.SQL[uk].Text:= OwnerData[Ord(uk)];

  if VarArrayHighBound(OwnerData, 1) > 3 then
  begin
    UpdateSQL.BeforeGetParam:= BeforeGetUserParam;
    for i:= 4 to VarArrayHighBound(OwnerData, 1) do
      with TParam(FUserParams.Add) do
      begin
        Name:= OwnerData[i][0];
        DataType:= OwnerData[i][1];
        ParamType:= OwnerData[i][2];
        Value:= OwnerData[i][3];
      end;
  end;
end;

end.
